import pycuda.autoinit
import pycuda.cumath
import pycuda.gpuarray as gpuarray
import numpy
import time
from pycuda.compiler import SourceModule
import pycuda.driver as drv

#We're going to test triangles! 
#We'll have two arrays o and t which are the opposite length and the angle
o_cpu = numpy.random.randn(10000).astype(numpy.float32)
t_cpu = numpy.random.randn(10000).astype(numpy.float32)
o_gpu = gpuarray.to_gpu(o_cpu)
t_gpu = gpuarray.to_gpu(t_cpu)

ts = time.time()

#Do the GPU calculations first
h_gpu = ((o_gpu/pycuda.cumath.sin(t_gpu)))
a_gpu = pycuda.cumath.sqrt((h_gpu*h_gpu)/(o_gpu*o_gpu))


print "time taken GPU", time.time() - ts

#now do it on the cpu
ts = time.time()
h_cpu = (o_cpu / numpy.sin(t_cpu))
a_cpu = numpy.sqrt((h_cpu*h_cpu)/(o_cpu*o_cpu))


print "time taken CPU", time.time() - ts

#Now to try a different way of doing things on the GPU

mod = SourceModule("""
__global__ void calculate_them(float *h, float *a, float *o, float *t)
{
  const int i = threadIdx.x + blockDim.x * blockIdx.x;
  h[i] = o[i]/sin(t[i]);
  a[i] = sqrt((h[i]*h[i])/(o[i]*o[i]));
}
""")

calculate_them = mod.get_function("calculate_them")

h_gpu2 = numpy.zeros_like(h_cpu)
a_gpu2 = numpy.zeros_like(a_cpu)

ts = time.time()

calculate_them(
        drv.Out(h_gpu2), drv.Out(a_gpu2), drv.In(o_cpu), drv.In(t_cpu),
        block=(1000,1,1), grid=(10,1,1))
        
print "time taken GPU2", time.time() - ts

#Print out the arrays so we can do a quick sanity check
print a_gpu
print a_cpu
print a_gpu2